#import "SpeechSynthesisPatch.h"
//#import "SpeechSynthesisPatchUI.h"
#import <AppKit/NSSpeechSynthesizer.h>

@implementation SpeechSynthesisPatch : QCPatch

/*+ (Class)inspectorClassWithIdentifier:(id)fp8
{
	return [SpeechSynthesisPatchUI class];
}*/

+ (int)executionMode
{
	return 1;
}
+ (BOOL)allowsSubpatches
{
	return NO;
}
+ (int)timeMode
{
	return 1;
}


- (id)initWithIdentifier:(id)fp8
{
	id z=[super initWithIdentifier:fp8];

	// if this isn't set, we blow up.  if it's too high, we get garbage menu items.
	// if it's too low, we chop off options.  so, we have to query the port for the number of items...
	// @@@ instead of querying the port like we should, we just query the speechsynthesis engine.  This will work, as long as the
	// value doesn't change.
	[inputVoice setMaxIndexValue:[[NSSpeechSynthesizer availableVoices] count]-1];
	speech = nil;

	return z;
}


- (id)setup:(id)fp8
{
	speechLock = [[NSLock alloc] init];
	oldValue = FALSE;
	return fp8;
}
- (void)cleanup:(id)fp8
{


}


- (void)enable:(id)fp8
{


}
- (void)disable:(id)fp8
{


}

- (BOOL)execute:(id)fp8 time:(double)fp12 arguments:(id)fp20
{
	//NSLog(@"execute: %i %i %i\n",[inputTrigger booleanValue], oldValue, speech);
	[speechLock lock];
	if([inputTrigger booleanValue] == TRUE && oldValue == FALSE && speech == nil)
	{
		// speak the text.
		//NSLog(@"trying to talk\n");
		speech = [[NSSpeechSynthesizer alloc] init];
		[speech setVoice:[[NSSpeechSynthesizer availableVoices] objectAtIndex:[inputVoice indexValue]]];
		[speech setUsesFeedbackWindow:FALSE];
		[speech setDelegate:self];
		[speech startSpeakingString:[inputString stringValue]];
	}
	if([speech isSpeaking] == NO)
	{
		[speech release];
		speech = nil;
	}
	[speechLock unlock];
	
	oldValue = [inputTrigger booleanValue];
		
	return YES;
}

- (void)speechSynthesizer:(NSSpeechSynthesizer *)sender didFinishSpeaking:(BOOL)success
{
	//NSLog(@"didFinishSpeaking\n");
	// clean up
	[speechLock lock];
		[speech release];
		speech = nil;
	[speechLock unlock];
}

- (void)speechSynthesizer:(NSSpeechSynthesizer *)sender willSpeakWord:(NSRange)wordToSpeak ofString:(NSString *)text
{
	//NSLog(@"willSpeakWord\n");
	++spokenWords;
	[outputWords setDoubleValue:(double)spokenWords];
}

- (void)speechSynthesizer:(NSSpeechSynthesizer *)sender willSpeakPhoneme:(short)phonemeOpcode
{
	[outputPhoneme setDoubleValue:(double)phonemeOpcode];
	//NSLog(@"willSpeakPhoeneme: %i\n",phonemeOpcode);
}

@end
